import 'dart:ui';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:line_awesome_flutter/line_awesome_flutter.dart';
import 'package:nui_animation/nui_animation.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_media/model/nui_media_list_repo.dart';
import 'package:nui_media/model/nui_media_models.dart';
import 'package:nui_media/nui_media.dart';
import 'package:nui_media/ui/nui_media_preview.dart';
import 'package:permission_handler/permission_handler.dart';

typedef MediaFileSelectionListener = void Function(NUIMediaFile added, NUIMediaFile removed, List<NUIMediaFile> selected);

class NUIMediaSelectionHorizontalStyle{
  NUIMediaSelectionHorizontalStyle({this.outlineColor, this.backgroundColor, this.accentColor, this.accentBrightColor, this.shadowColor, this.cameraIcon});
  Color outlineColor;
  Color backgroundColor;
  Color accentColor;
  Color accentBrightColor;
  Color shadowColor;
  Widget cameraIcon;

  Widget getCameraIcon() => cameraIcon ?? Icon(LineAwesomeIcons.camera, size: 24, color: getAccentColor());
  Color getShadowColor() => shadowColor ?? NUIColors.NUIBlack.withOpacity(0.5);
  Color getOutlineColor() => outlineColor ?? NUIColors.NUIGrayOutline;
  Color getBackgroundColor() => backgroundColor ?? NUIColors.NUIWhite;
  Color getAccentColor() => accentColor ?? NUIColors.NUIAccent;
  Color getAccentBrightColor() => accentBrightColor ?? NUIColors.NUIAccent;
}

class NUIListItemImageHorizontalSkeletal extends NUIListStatefulWidget {
  NUIListItemImageHorizontalSkeletal({this.style, NUIListActionController handler, this.widthRatio: 1}): super(controller: handler);
  final NUIMediaSelectionHorizontalStyle style;
  final double widthRatio;
  @override
  _NUIListItemImageHorizontalSkeletalState createState() => _NUIListItemImageHorizontalSkeletalState();
}

class _NUIListItemImageHorizontalSkeletalState extends NUIListState<NUIListItemImageHorizontalSkeletal> {
  @override
  Widget build(BuildContext context) {
    final style = widget.style ?? getConfigData("style") as NUIMediaSelectionHorizontalStyle ?? NUIMediaSelectionHorizontalStyle();

    return Container(
      alignment: Alignment.center,
      width: 72 * widget.widthRatio ?? 1,
      height: 72,
      decoration: NUIBoxDecoration(
        color: style.getOutlineColor(),
        borderRadiusCircular: 10,
        shadowRadius: 4,
        shadowOffset: Offset(1, 1),
        shadowColor: style.getShadowColor()
      ),
      child: Container(
        width: 72 * widget.widthRatio ?? 1,
        height: 72,
        decoration: NUIBoxDecoration(
          color: style.getBackgroundColor(),
          borderRadiusCircular: 10,
          shadowRadius: 0,
        ),
        child: NUIClipView(
          borderRadius: BorderRadius.circular(10),
          child: NUISkeletonContainer(
            baseColor: NUIColors.NUISkeletalLightGray,
            highlightColor: NUIColors.NUISkeletalGray,
            child: NUISkeletonView(
              width: 72 * widget.widthRatio ?? 1,
              height: 72,
            )
          ),
        ),
      ),
    );
  }
}

class NUIListItemImageHorizontal extends NUIListStatefulWidget {
  NUIListItemImageHorizontal({@required this.file, this.style, NUIListActionController handler}): super(controller: handler);
  final NUIMediaHorizontalSelection file;
  final NUIMediaSelectionHorizontalStyle style;
  @override
  _NUIListItemImageHorizontalState createState() => _NUIListItemImageHorizontalState();
}

class _NUIListItemImageHorizontalState extends NUIListState<NUIListItemImageHorizontal> {
  @override
  Widget build(BuildContext context) {
    final style = widget.style ?? getConfigData("style") as NUIMediaSelectionHorizontalStyle ?? NUIMediaSelectionHorizontalStyle();
    final selected = getCheckedData(widget.file.file.path) != null;

    return NUIBouncingAnimation(
      scaleFactor: 0.5,
      onTap: (){
//        if(selected){
//          getController().updateCheckedData((map) {
//            map.remove(widget.file.file.path);
//          });
//          getController().onItemClick(widget.file, NUIListActions.REMOVE);
//        }
//        else{
//          getController().updateCheckedData((map) {
//            map[widget.file.file.path] = widget.file.file.path;
//          });
          getController().onItemClick(widget.file, NUIListActions.NORMAL);
//        }
      },
      onLongPress: (){
        getController().onItemClick(widget.file, NUIListActions.LONG);
      },
      child: Container(
        alignment: Alignment.center,
        width: 72,
        height: 72,
        decoration: NUIBoxDecoration(
          color: style.getBackgroundColor(),
          borderRadiusCircular: 10,
          shadowRadius: 3,
          shadowOffset: Offset(1, 1),
          shadowColor: selected != true ? style.getShadowColor() : style.accentColor.withOpacity(0.4)
        ),
        child: Container(
          width: 72,
          height: 72,
          child: NUIClipView(
            borderRadius: BorderRadius.circular(10),
            child: Stack(
              children: [
                widget.file.file.thumbData == null
                    ? loadImageLocal(widget.file.file.path, 72, 72)
                    : NUIFutureWidget(
                        futureValue: widget.file.file.thumbData,
                        callback: (state, data, message){
                          if(state == FutureState.SUCCESS) return loadImageLocal(widget.file.file.path, 70, 70, thumbdata: data);
                          return SizedBox(width: 72, height: 72);
                        }),
                SizedBox(width: 72, height: 72),
                Visibility(
                  visible: selected,
                  child: Container(
                    decoration: NUIBoxDecoration(
                      color: style.accentColor.withOpacity(0.4)
                    ),
                    width: 72,
                    height: 72,
                    alignment: Alignment.bottomRight,
                    padding: EdgeInsets.all(3),
                    child: NUIClipView(
                      borderRadius: BorderRadius.circular(10),
                      child: BackdropFilter(
                        filter: ImageFilter.blur(sigmaX: 5.0, sigmaY: 5.0),
                        child: Container(
                            height: 26,
                            width: 26,
                            alignment: Alignment.center,
                            decoration: NUIBoxDecoration(
                                color: NUIColors.NUIBlack.withOpacity(0.1)
                            ),
                            child: Icon(LineAwesomeIcons.check, size: 18, color: NUIColors.NUIWhite)
                        ),
                      ),
                    ),
                  ),
                ),
                Visibility(
                  child: Positioned(
                    top: 3,
                    left: 3,
                    child: NUIClipView(
                      borderRadius: BorderRadius.circular(10),
                      child: BackdropFilter(
                        filter: ImageFilter.blur(sigmaX: 5.0, sigmaY: 5.0),
                        child: Container(
                            height: 26,
                            width: 26,
                            alignment: Alignment.center,
                            decoration: NUIBoxDecoration(
                                color: NUIColors.NUIBlack.withOpacity(0.1)
                            ),
                            child: Icon(LineAwesomeIcons.play, size: 18, color: NUIColors.NUIWhite)
                        ),
                      ),
                    ),
                  ),
                  visible: widget.file.file.thumbData != null,
                ),
              ],
            ),
          ),
        ),
      ),
    );
  }
}

class NUIListItemCameraHorizontal extends NUIListStatefulWidget {
  NUIListItemCameraHorizontal({this.style, NUIListActionController handler}): super(controller: handler);
  final NUIMediaSelectionHorizontalStyle style;
  @override
  _NUIListItemCameraHorizontalState createState() => _NUIListItemCameraHorizontalState();
}

class _NUIListItemCameraHorizontalState extends NUIListState<NUIListItemCameraHorizontal> {
  @override
  Widget build(BuildContext context) {
    final style = widget.style ?? getConfigData("style") as NUIMediaSelectionHorizontalStyle ?? NUIMediaSelectionHorizontalStyle();

    return NUIBouncingAnimation(
      scaleFactor: 0.5,
      onTap: (){
        getController().onItemClick("Take Photo", NUIListActions.ADD);
      },
      child: Container(
//      margin: EdgeInsets.only(right: 10),
        alignment: Alignment.center,
        width: 72,
        height: 72,
        decoration: NUIBoxDecoration(
          color: style.getBackgroundColor(),
          borderRadiusCircular: 10,
          shadowRadius: 3,
          shadowOffset: Offset(1, 1),
          shadowColor: style.getShadowColor()
        ),
        child: Container(
          width: 72,
          height: 72,
          child: Align(
            alignment: Alignment.center,
            child: style.getCameraIcon(),
          ),
        ),
      ),
    );
  }
}

typedef NUIMediaFileCallback = void Function(List<NUIMediaFile> files);
class NUIMediaSelectionController{
  Function _takeNewPhotoCallback;
  Function _recordNewVideoCallback;
  NUIMediaFileCallback _addNewFileCallback;
  Function _selectFromGallery;

  void takeNewPhoto(){
    if(_takeNewPhotoCallback != null) _takeNewPhotoCallback();
  }
  void recordNewVideo(){
    if(_recordNewVideoCallback != null) _recordNewVideoCallback();
  }
  void addNewFiles(List<NUIMediaFile> files){
    if(_addNewFileCallback != null) _addNewFileCallback(files);
  }
  void selectFromGallery(){
    if(_selectFromGallery != null) _selectFromGallery();
  }

  void _dispose(){
    _takeNewPhotoCallback = null;
    _recordNewVideoCallback = null;
    _addNewFileCallback = null;
    _selectFromGallery = null;
  }
}

class NUIMediaSelectionHorizontalView extends StatefulWidget {
  NUIMediaSelectionHorizontalView({this.limit, this.style, this.option: NUIMediaGalleryOption.BOTH, this.maximumSelection, this.controller, this.showCamera: true, this.listener});
  final NUIMediaSelectionHorizontalStyle style;
  final NUIMediaGalleryOption option;
  final bool showCamera;
  final int limit;
  final int maximumSelection;
  final NUIMediaSelectionController controller;
  final MediaFileSelectionListener listener;

  @override
  _NUIMediaSelectionHorizontalViewState createState() => _NUIMediaSelectionHorizontalViewState();
}

class _NUIMediaSelectionHorizontalViewState extends State<NUIMediaSelectionHorizontalView> {
  NUIListViewBloc<NUIMediaHorizontalSelection, NUIMediaHorizontalSelection> bloc;

  NUIDialog curShowingPreview;
  NUIListController listController = NUIListController();
  List<NUIMediaHorizontalSelection> selected = List();
  Map<String, String> selectedMap = Map();
  NUIMediaSelectionController controller;
  Map<String, int> filesIndex = Map();

  @override
  void initState(){
    super.initState();
    controller = widget.controller ?? NUIMediaSelectionController();
    bloc = NUIListViewBloc<NUIMediaHorizontalSelection, NUIMediaHorizontalSelection>(callbackMethod: (List<NUIMediaHorizontalSelection> current, bool isFirst, bool append, NUIMediaHorizontalSelection cursor) async{
      final hasGalleryPermission = await NUIPermissionUtil.requestPermissions([Permission.storage, Permission.mediaLibrary, Permission.photos]);
      if(hasGalleryPermission) {
        final imagesFuture = NUIMedia.get().getFromGallery(option: widget.option, limit: widget.limit ?? 200);
        await NUIAsync.delayForResult(Duration(milliseconds: 500), () => null);
        final images = await imagesFuture;
        int counter = 0;
        final result = ListResult<NUIMediaHorizontalSelection, NUIMediaHorizontalSelection>(data: images.map((e){
          final mediaFile = NUIMediaHorizontalSelection(file: e);
          filesIndex[mediaFile.file.path] = counter;
          counter++;
          return mediaFile;
        }).toList(), success: true, message: "Get media selections successful", cursor: null);
        return result;
      }
      else{
        final result = ListResult<NUIMediaHorizontalSelection, NUIMediaHorizontalSelection>(data: [], success: false, message: "Failed to get file selections", cursor: null);
        return result;
      }
    });

    controller._addNewFileCallback = (files){
      if(!isNullOrEmpty(files)){
        _addMultipleFile(files);
      }
    };
    controller._recordNewVideoCallback = () async{
      final file = await NUIMediaUtil.recordVideo(context);
      if(file != null){
        _addSingleFile(file);
      }
    };
    controller._takeNewPhotoCallback = () async{
      final file = await NUIMediaUtil.takePhoto(context);
      if(file != null){
        _addSingleFile(file);
      }
    };
    controller._selectFromGallery = () async{

    };
  }

  void _addMultipleFile(List<NUIMediaFile> files, {bool showPreview : true}) async{
    for(NUIMediaFile file in files) {
      if (!selectedMap.containsKey(file.path)) {
        selected.add(NUIMediaHorizontalSelection(file: file));
      }
      selectedMap[file.path] = file.path;
    }

    if(showPreview != false) {
      final files = await NUIMedia.get().showFileSelection(
          context,
          type: widget.option,
          maximumSelection: widget.maximumSelection,
          accentColor: widget.style.getAccentColor(),
          accentColorBright: widget.style.getAccentBrightColor(),
          selections: selected.map((e) => e.file).toList()
      );
      if(files != null) {
        _refreshFiles(files);
      }
    }
    else{
      _refreshFiles(files);
    }
  }

  void _addSingleFile(NUIMediaFile file, {bool showPreview : true}) async{
    if(!selectedMap.containsKey(file.path)) {
      selected.add(NUIMediaHorizontalSelection(file: file));
    }
    selectedMap[file.path] = file.path;

    if(showPreview != false) {
      final files = await NUIMedia.get().showFileSelection(
          context,
          type: widget.option,
          maximumSelection: widget.maximumSelection,
          accentColor: widget.style.getAccentColor(),
          accentColorBright: widget.style.getAccentBrightColor(),
          selections: selected.map((e) => e.file).toList()
      );
      if(files != null) {
        _refreshFiles(files);
      }
    }
    else{
      _refreshFiles([file]);
    }
  }

  void _refreshFiles(List<NUIMediaFile> files){
    selectedMap.clear();
    selected.clear();

    listController.updateCheckedMap((map) {
      map.clear();
      for(NUIMediaFile file in files) {
        map[file.path] = file.path;
      }
    });

    final toInsertList = List<NUIMediaHorizontalSelection>();
    for(NUIMediaFile file in files){
      if(!selectedMap.containsKey(file.path)){
        selected.add(NUIMediaHorizontalSelection(file: file));
        selectedMap[file.path] = file.path;
      }

      if(filesIndex.containsKey(file.path)) continue;
      toInsertList.add(NUIMediaHorizontalSelection(file: file));
      filesIndex[file.path] = 0;
    }
    listController.insertItems(toInsertList, append: false);
    _notifyNewFilesSelection(selected.map((e) => e.file).toList());
  }

  void _notifyNewFilesSelection(List<NUIMediaFile> files, {NUIMediaFile added, NUIMediaFile removed}){
    if(widget.listener != null) widget.listener(added, removed, selected.map((e) => e.file).toList());
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      height: 98,
      child: NUIAnimatedHeaderList(
        bloc: bloc,
        repo: NUIMediaListItemRepo(),
        enablePagination: false,
        enableSwipeRefresh: false,
        skeletalCount: 2,
        controller: listController,
        skeletalInflater: NUILoadingListItem(inflater: (position) => NUIListItemImageHorizontalSkeletal(style: widget.style, widthRatio: position == 0 ? 1.5 : 1.0), shadowRadius: 4),
        configMap: {
          "style": widget.style ?? NUIMediaSelectionHorizontalStyle()
        },
        orientation: Axis.horizontal,
        padding: const EdgeInsets.only(left: 15, top: 10, bottom: 10, right: 15),
        fixedFirstItem: widget.showCamera == true ? (() => NUIMediaHorizontalCameraSelection()) : null,
        initialList: List<NUIMediaHorizontalSelection>(),
        clickListener: (data, position, action) async{
          if(match(action, NUIListActions.NORMAL)){
            final image = data as NUIMediaHorizontalSelection;
            _addSingleFile(image.file);
          }
          else if(match(action, NUIListActions.LONG)){
            final image = data as NUIMediaHorizontalSelection;
            curShowingPreview = showMediaPagerPreview(context, files: [image.file]);
          }
          else if(match(action, NUIListActions.ADD)){
            final file = await NUIMediaUtil.takePhoto(context);
            if(file != null){
              _addSingleFile(file);
            }
          }
        },
      ),
    );
  }

  @override
  void dispose(){
    super.dispose();
    controller?._dispose();
  }
}