import 'package:better_player/better_player.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_media/model/nui_media_file.dart';
import 'package:nui_media/nui_media.dart';
import 'package:nui_media/nui_media_util.dart';
import 'package:nui_media/ui/nui_media_preview.dart';

class NUIVideoListViewPlayer extends StatefulWidget {
  NUIVideoListViewPlayer({@required this.mediaFile, this.configuration, this.enableFullScreen: true, this.boxFit: BoxFit.fitWidth, this.mute: true, this.autoPlay:true, this.playFraction:0.8, this.showButtons:false, this.aspectRatio, this.loop: true, this.loader, this.width, this.height});
  final NUIMediaFile mediaFile;
  final BetterPlayerConfiguration configuration;
  final bool autoPlay;
  final double playFraction;
  final bool showButtons;
  final bool enableFullScreen;
  final double aspectRatio;
  final double width;
  final double height;
  final Widget loader;
  final bool loop;
  final bool mute;
  final BoxFit boxFit;
  @override
  _NUIVideoListViewPlayerState createState() => _NUIVideoListViewPlayerState();
}

class _NUIVideoListViewPlayerState extends NUIVideoPlayerState<NUIVideoListViewPlayer> {
  BetterPlayerDataSource dataSource;
  BetterPlayerListVideoPlayerController _controller;
  BetterPlayerController _playerController;
  BetterPlayerConfiguration _configuration;

  @override
  void initState(){
    super.initState();
    _controller = BetterPlayerListVideoPlayerController();
    _controller.setVolume(widget.mute == true ? 0.0 : 1.0);
    final placeHolder = widget.loader ?? Center(child: SizedBox(height: 30.0, width: 30.0, child: CircularProgressIndicator(strokeWidth: 1.5, valueColor: new AlwaysStoppedAnimation<Color>(NUIColors.NUIDarkGrayOutline), backgroundColor: NUIColors.NUISkeletalGray)));
    _configuration = widget.configuration ??
        BetterPlayerConfiguration(
          fit: widget.boxFit ?? BoxFit.fitWidth,
          placeholder: placeHolder,
//          showControlsOnInitialize: widget.showButtons,
          aspectRatio: widget.aspectRatio,
          autoPlay: widget.autoPlay,
          looping: widget.loop,
          controlsConfiguration: getNUIStyleControlsConfiguration(widget.showButtons ?? true, widget.enableFullScreen ?? true)
        );

    dataSource = NUIMediaUtil.dataSourceFromNUIMedia(widget.mediaFile);
    _playerController = BetterPlayerController(_configuration, betterPlayerDataSource: dataSource);
    _playerController?.setVolume(0.0);
  }

  @override
  Widget build(BuildContext context) {
//    if(widget.mute == true){
//      NUIAsync.delay(Duration(milliseconds: 1000), () {
//        _playerController?.setVolume(0.0);
//      });
//    }
    return adjustSize(mediaFile: widget.mediaFile, adjustSize: widget.aspectRatio == null, width: widget.width, height: widget.height);
  }

  @override
  Widget getVideoPlayer() {
    _controller.setBetterPlayerController(_playerController);
    _controller.setVolume(0.0);
    return BetterPlayerListVideoPlayer(dataSource, betterPlayerListVideoPlayerController: _controller, configuration: _configuration, playFraction: widget.playFraction, autoPlay: widget.autoPlay, autoPause: widget.autoPlay);
  }
}
